;
;       PROGRAMA PARA QUE ROBOT DETECTE LA PRESENCIA DE LA PRESA.
;
;       Sensores: - Un solo detector digital para deteccin de la presa.
;                 - Cuatro sensores analgicos para deteccin de posibles
;                 obstculos formados por emisores() y receptores().
;                 - Dos bumpers frontales.
;
;       Descripcin: El robot busca a la presa girando sobre si mismo hacia
;   la derecha hasta completar una vuelta y si no lo detecta se para. La programacin se ha
;   hecho a base de temporizacin por lo que si el robot gira en busca de la
;   presa y detecta un obstculo gira 0.5 seg. a la izquierda y otros 0.5 seg.
;   hacia delante.
;       
PORTA   EQU $00    ; Puerto desde el que activan los motores
PORTD   EQU $08
PORTE   EQU $0A
DDRD    EQU $09
TMSK1   EQU $22
TCTL2   EQU $21
TFLG1   EQU $23    ; Flags de los temporizadores
TCNT    EQU $0E    ; Lleva la cuenta actual del temporizador
TOC1    EQU $16    ; Registro que almacena el tiempo final
ACUM    EQU $00E8

        ORG $0000
        
paso_s1 RMB 2
paso_s2 RMB 2

        ORG $B600

inic    LDAA #$7E
        STAA ACUM
        LDX #rut_enc
        STX ACUM+1
        BSR borr_s1
        LDX #$1000
        BCLR DDRD,X $00


inicte  LDAB #$80        ; Inicaliza el flag de interrupcin de OC1
        STAB TFLG1,X

ini_enc BSET TCTL2,X $30
        BSET TMSK1,X $04
        CLI

guard   BRCLR PORTD,X $08 go_der
        BRCLR PORTD,X $10 go_izq
        BRCLR PORTD,X $04 go_rec
        BSR borr_s1
        BSR gira_de
guard1  BRCLR PORTE,X $04 go_detod
        BRCLR PORTD,X $08 go_der
        BRCLR PORTD,X $10 go_izq
        BRCLR PORTD,X $04 go_rec
        LDD paso_s1
        CPD #64
        BNE guard1
        BSR borr_s1
        BSR sig_rec
guard2  BRCLR PORTE,X $01 testbd
        BRCLR PORTE,X $02 testbi
        BRCLR PORTD,X $08 go_der
        BRCLR PORTD,X $10 go_izq
        BRCLR PORTD,X $04 go_rec
        LDD paso_s1
        CPD #48
        BNE guard2
        CLRA
        STAA PORTA,X
        LDY #200
guard3  BRCLR PORTD,X $08 go_der
        BRCLR PORTD,X $10 go_izq
        BRCLR PORTD,X $04 go_rec
        BSR temp
        DEY
        BNE guard3
        BRA guard

borr_s1 CLRA
        CLRB
        STD paso_s1
        RTS

borr_s2 CLRA
        CLRB
        STD paso_s2
        RTS

go_izq  JMP izquier

go_der  JMP derecha

go_rec  JMP avanza

go_detod JMP detobsd

go_detoi JMP detobsi

gira_de LDAA #$38
        STAA PORTA,X
        RTS

sig_rec LDAA #$18
        STAA PORTA,X
        RTS

gira_iz LDAA #$58
        STAA PORTA,X
        RTS

testbd  JMP bumpd
testbi  JMP bumpi
        

temp    LDD TCNT,X                  ; Temporizacin de 10 ms.
        ADDD #20000
        STD TOC1,X                  ; Carga TOC1 con cuenta final (10 ms)
loop    BRCLR TFLG1,X $80 loop      ; Esperamos a que pasen 10 ms
        BCLR TFLG1,X $7F            ; Desactivamos OC1F para siguiente cuenta
        RTS

esp_det LDY #20
wait    BSR temp
        DEY
        BNE wait
        RTS

rut_enc   LDD paso_s1
          ADDD #1
          STD paso_s1
          LDD paso_s2
          ADDD #1
          STD paso_s2
          LDAA #$04
          STAA TFLG1,X
          RTI

izquier JSR gira_iz
        BRCLR PORTE,X $08 detobsi
        JSR esp_det
        JMP guard

derecha JSR gira_de
        BRCLR PORTE,X $04 detobsd
        JSR esp_det
        JMP guard

avanza  JSR sig_rec
        BRCLR PORTD,X $20 go_fin
        BRCLR PORTE,X $01 testbd
        BRCLR PORTE,X $02 testbi
        JSR esp_det
        JMP guard

detobsd JSR gira_iz
        BRA rut_gi

detobsi JSR gira_de
rut_gi  JSR borr_s1
giro    LDD paso_s1
        CPD #4
        BNE giro
recto   JSR sig_rec
        JSR borr_s1
recto1  BRCLR PORTE,X $01 testbd
        BRCLR PORTE,X $02 testbi
        LDD paso_s1
        CPD #6
        BNE recto1
        JMP guard

go_fin  CLRA
        STAA PORTA,X
        LDY #500
esp_fin JSR temp
        DEY
        BNE esp_fin
        JMP fin

dere    JSR gira_de
        JSR borr_s1
de1     LDD paso_s1
        CPD #20
        BEQ fin_fi
        BRCLR PORTE,X $02 bumpi
        BRSET PORTE,X $08 de1
        BRA fin_de

izqui   JSR gira_iz
        JSR borr_s1
iz1     LDD paso_s1
        CPD #20
        BEQ fin_fd
        BRCLR PORTE,X $01 bumpd
        BRSET PORTE,X $04 iz1
        BRA fin_iz 

frented JSR borr_s2
        JSR sig_rec
fd1     LDD paso_s2
        CPD #50
        BEQ fin_fd
        BRCLR PORTE,X $01 bumpd
        BRCLR PORTE,X $04 fd1
        JMP fin_fd

frentei JSR borr_s2
        JSR sig_rec
fi1     LDD paso_s2
        CPD #50
        BEQ fin_fi
        BRCLR PORTE,X $02 bumpi
        BRCLR PORTE,X $08 fi1
        JMP fin_fi

bumpd     BSR atras
          BRA izqui
fin_iz    LDY #30
espera1   JSR temp
          DEY
          BNE espera1
          BRA frented
fin_fd    BRA pasa_obst
          

bumpi     BSR atras                     ; Si no se activan los dos el objeto slo a izq.
          BRA dere
fin_de    LDY #30
espera    JSR temp
          DEY
          BNE espera
          BRA frentei
fin_fi    BRA pasa_obst

atras   LDAA #$78                     ; Atrs durante un tiempo fijo
        STAA PORTA,X
        JSR borr_s1
a1      LDD paso_s1
        CPD #2
        BNE a1
        RTS

pasa_obst JSR sig_rec
          JSR borr_s1
sal_obst  BRCLR PORTE,X $01 bumpd
          BRCLR PORTE,X $02 bumpi
          LDD paso_s1
          CPD #28
          BNE sal_obst
          JMP guard
          
fin       END


